<?php

namespace Drupal\search_api_opensolr\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\key\KeyInterface;
use Drupal\key\KeyRepositoryInterface;
use Drupal\search_api_opensolr\Traits\OpensolrKeyDependencyTrait;

/**
 * Retrieves and processes the global opensolr configuration.
 */
class OpenSolrConfig {
  use OpensolrKeyDependencyTrait;

  /**
   * The opensolr configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The key repository service if the key module is installed.
   *
   * @var false|\Drupal\key\KeyRepositoryInterface
   */
  protected false|KeyRepositoryInterface $keyRepositoryService;

  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->config = $config_factory->get('search_api_opensolr.opensolrconfig');
    $this->keyRepositoryService = $this->keyRepositoryService();
  }

  /**
   * Gets the API credentials from the stored configuration.
   *
   * @return array
   *   Returns an array with the email and api_key keys and values.
   */
  public function getApiCredentials() {
    return $this->extractApiCredentials($this->config->get('opensolr_credentials'));
  }

  /**
   * Extracts the API credentials from the given array.
   *
   * @param array $credentials
   *   An array of credentials that should contain the email, the api_key and
   *   the api_key_raw. The api_key is available when using the key module, the
   *   other one is used when key is not available.
   *
   * @return array
   *   Returns an array with the email and the api_key.
   */
  public function extractApiCredentials(array $credentials) {
    if ($this->keyRepositoryService) {
      // Override with key credential if it exists.
      $key = $this->keyRepositoryService->getKey($credentials['api_key']);
      if ($key instanceof KeyInterface) {
        $credentials['api_key'] = $key->getKeyValue();
      }
    }
    else {
      $credentials['api_key'] = $credentials['api_key_raw'];
      unset($credentials['api_key_raw']);
    }
    return $credentials;
  }

}
